"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var PATH = require("path");
var child_process_1 = require("child_process");
var Run = /** @class */ (function () {
    // ########################################
    function Run(consoleArguments, commandSettings, currentRootPath, rulesetValidation) {
        this.consoleArguments = consoleArguments;
        this.commandSettings = commandSettings;
        this.currentRootPath = currentRootPath;
        this.rulesetValidation = rulesetValidation;
        // ########################################
        this.sniffersTSConfigFilePath = '';
        this.params = {};
        this.filePath = '';
        this.statusToOutput = false;
    }
    // ########################################
    Run.prototype.process = function () {
        var _this = this;
        this.init();
        var relativeRulesetPath = PATH.resolve(this.params[Run.RULESET_FLAG]);
        var relativePathToCommand = PATH.resolve(Run.HTMLHINT_NODE_COMMAND);
        var relativePathToTSCCommand = PATH.resolve(Run.TSC_NODE_COMMAND);
        if (!this.rulesetValidation.isValid(relativeRulesetPath)) {
            var outputError_1 = '';
            this.rulesetValidation.getErrorsMessages().forEach(function (error) {
                outputError_1 += error + '\n';
            });
            throw new Error(outputError_1);
        }
        var rulesetData = this.getDataFromRuleset(relativeRulesetPath);
        var relativePathToRulesdir = PATH.resolve(rulesetData.rulesdir);
        this.createRulesFile(rulesetData.rules);
        var includeFolders = '';
        if (rulesetData.include) {
            includeFolders = rulesetData.include.join(' ');
        }
        this.compileSniffers(relativePathToTSCCommand)
            .then(function () {
            return _this.commandExec(relativePathToCommand, _this.filePath, relativePathToRulesdir, includeFolders);
        })
            .then(function (result) {
            _this.removeFile(_this.filePath);
            _this.removeCompiledSniffers();
            process.stdout.write(result);
        })
            .catch(function (error) {
            _this.removeFile(_this.filePath);
            _this.removeCompiledSniffers();
            process.stderr.write("" + error);
        });
    };
    // ########################################
    Run.prototype.init = function () {
        var _this = this;
        this.consoleArguments.forEach(function (arg) {
            if (arg.includes('--status-to-output')) {
                _this.statusToOutput = true;
            }
            if (arg.includes(Run.RULESET_FLAG)) {
                _this.params[Run.RULESET_FLAG] = _this.getArgumentByName(arg);
                return;
            }
        });
        this.sniffersTSConfigFilePath = PATH.resolve(this.currentRootPath + '/../' + Run.sniffersTSConfigFileName);
    };
    Run.prototype.commandExec = function (command, ruleset, customRulesDir, includeFolders) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            child_process_1.exec(command + " --config " + ruleset + " --rulesdir " + customRulesDir + " " + includeFolders + " || exit 0", function (error, stdout, stderr) {
                if (error) {
                    var createOutput = _this.printOutput(error, 'Error');
                    var output = "" + createOutput;
                    reject(output);
                }
                if (stdout) {
                    if (stdout.includes('no errors found')) {
                        var createOutput = _this.printOutput('HTML validated successfully!', 'Success');
                        var output = "" + createOutput;
                        resolve(output);
                    }
                    else {
                        var createOutput = _this.printOutput(stdout, 'Error');
                        var output = "" + createOutput;
                        reject(output);
                    }
                }
                if (stderr) {
                    var createOutput = _this.printOutput(stderr, 'Error');
                    var output = "" + createOutput;
                    reject(output);
                }
            });
        });
    };
    // ########################################
    Run.prototype.getDataFromRuleset = function (rulesetPath) {
        var data = fs.readFileSync(rulesetPath);
        return JSON.parse(data);
    };
    Run.prototype.compileSniffers = function (relativePathToCommand) {
        var relativePathToConfig = PATH.resolve(this.sniffersTSConfigFilePath);
        return new Promise(function (resolve, reject) {
            child_process_1.exec(relativePathToCommand + " -p " + relativePathToConfig + " || exit 0", function (error, stdout, stderr) {
                if (error) {
                    reject(error);
                }
                if (stderr) {
                    reject(stderr);
                }
                else {
                    resolve(true);
                }
            });
        });
    };
    Run.prototype.createRulesFile = function (rules) {
        var ruleFolderPath = this.params[Run.RULESET_FLAG].slice(0, this.params[Run.RULESET_FLAG].lastIndexOf('/') + 1);
        this.filePath = PATH.resolve("" + ruleFolderPath + Run.RULES_FILE_NAME);
        fs.writeFileSync(this.filePath, JSON.stringify(rules));
    };
    Run.prototype.removeFile = function (path) {
        fs.unlinkSync(path);
    };
    Run.prototype.removeCompiledSniffers = function () {
        var tsconfigSniffers = fs.readFileSync(this.sniffersTSConfigFilePath);
        var tsconfigSniffersData = JSON.parse(tsconfigSniffers);
        tsconfigSniffersData.include.forEach(function (folder) {
            if (fs.existsSync(folder)) {
                fs.readdirSync(folder).forEach(function (file) {
                    if (file.substring(file.length - 2) === 'js') {
                        var curPath = folder + "/" + file;
                        fs.unlinkSync(curPath);
                    }
                });
            }
            else {
                return;
            }
        });
    };
    // ########################################
    Run.prototype.getArgumentByName = function (paramStr) {
        var splittingParam = paramStr.split('=');
        return splittingParam[1];
    };
    Run.prototype.printOutput = function (message, status) {
        if (this.statusToOutput) {
            var outputObject = {
                status: status,
                output: message
            };
            return JSON.stringify(outputObject);
        }
        else {
            return message;
        }
    };
    // ########################################
    Run.HTMLHINT_NODE_COMMAND = "./node_modules/.bin/htmlhint";
    Run.TSC_NODE_COMMAND = './node_modules/.bin/tsc';
    Run.RULES_FILE_NAME = 'html-rules.json';
    Run.RULESET_FLAG = '--ruleset';
    Run.sniffersTSConfigFileName = 'tsconfig.sniffers.json';
    return Run;
}());
exports.Run = Run;
